"""
Collection of fit setups for dedicated (new) fit approaches.

Felix Metzner 2023
"""

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups.fit_setup_utilities import FitSetupTriplet

# R(D(*)) fit Setups
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import rdstar_fit_info

# Tag Calibration fit Setups

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_main_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_main_two_bkg_fit_info

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_one_bkg_floating_df_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_one_bkg_coupled_df_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_one_bkg_coupled_d_ds_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_one_bkg_extra_df_corr_fit_info

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_combined_bkg_floating_cf_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_combined_bkg_coupled_df_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_combined_bkg_coupled_d_ds_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_combined_bkg_floating_df_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_combined_bkg_extra_df_corr_fit_info

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import (
    tag_cal_split_bkg_floating_cf_fit_info,
    tag_cal_split_bkg_floating_df_fit_info,
    tag_cal_split_bkg_coupled_df_fit_info,
    tag_cal_split_bkg_coupled_d_ds_fit_info,
)

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import (
    tag_cal_per_tag_mode_combined_bkg_coupled_df_fit_info,
    tag_cal_per_tag_mode_combined_bkg_floating_ds_fit_info,
    tag_cal_per_tag_mode_charge_split_coupled_d_fit_info,
    tag_cal_per_tag_mode_charge_split_floating_ds_fit_info,
    tag_cal_per_tag_mode_b_charge_coupled_d_fit_info,
    tag_cal_per_tag_mode_b_charge_floating_ds_fit_info,
)

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_closure_a_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import tag_cal_closure_b_fit_info

# D** Calibration fit Setups
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import dss_cal_first_approach_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import dss_cal_second_approach_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import dss_cal_third_approach_fit_info

# Continuum Calibration fit Setups
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import cont_cal_first_approach_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import cont_cal_second_approach_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import cont_cal_third_approach_fit_info
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import cont_cal_fourth_approach_fit_info

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import cont_cal_in_b_tag_modes_fit_info

# D** Continuum Calibration fit Setups
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import cont_cal_in_dss_1st_approach_fit_info

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import cont_cal_in_b_tag_modes_in_dss_fit_info


__all__ = [
    "FitSetupCollection",
]


class FitSetupCollection:
    # Main Setups (the ones that are used...)
    rdstar_fit_setup: FitSetupTriplet = rdstar_fit_info.rdstar_fit_setup

    #
    # All RDStar Setups
    rdstar_fit_setup1: FitSetupTriplet = rdstar_fit_info.rdstar_fit_setup

    #
    # All Tag Calibration Setups
    # Used FEI Tag Calibration setups:
    tc_main__fs: FitSetupTriplet = tag_cal_main_fit_info.tag_cal_fit_setup
    tc_main_2b__fs: FitSetupTriplet = tag_cal_main_two_bkg_fit_info.tag_cal_fit_setup
    #
    # One BKG Component including B -> D l nu UP FEED into BKG for B -> D* l reco channels
    tc_one_bkg_floating_df__fs: FitSetupTriplet = tag_cal_one_bkg_floating_df_fit_info.tag_cal_fit_setup
    tc_one_bkg_coupled_df__fs: FitSetupTriplet = tag_cal_one_bkg_coupled_df_fit_info.tag_cal_fit_setup
    tc_one_bkg_coupled_d_ds__fs: FitSetupTriplet = tag_cal_one_bkg_coupled_d_ds_fit_info.tag_cal_fit_setup
    tc_one_bkg_extra_df_corr__fs: FitSetupTriplet = tag_cal_one_bkg_extra_df_corr_fit_info.tag_cal_fit_setup
    # Combined BKG Component WITHOUT B -> D l nu UP FEED
    tc_combined_bkg_floating_cf__fs: FitSetupTriplet = tag_cal_combined_bkg_floating_cf_fit_info.tag_cal_fit_setup
    tc_combined_bkg_floating_df__fs: FitSetupTriplet = tag_cal_combined_bkg_floating_df_fit_info.tag_cal_fit_setup
    tc_combined_bkg_coupled_df__fs: FitSetupTriplet = tag_cal_combined_bkg_coupled_df_fit_info.tag_cal_fit_setup
    tc_combined_bkg_coupled_d_ds__fs: FitSetupTriplet = tag_cal_combined_bkg_coupled_d_ds_fit_info.tag_cal_fit_setup
    tc_combined_bkg_extra_df_corr__fs: FitSetupTriplet = tag_cal_combined_bkg_extra_df_corr_fit_info.tag_cal_fit_setup
    # Split BKG Components
    tc_split_bkg_floating_cf__fs: FitSetupTriplet = tag_cal_split_bkg_floating_cf_fit_info.tag_cal_fit_setup
    tc_split_bkg_floating_df__fs: FitSetupTriplet = tag_cal_split_bkg_floating_df_fit_info.tag_cal_fit_setup
    tc_split_bkg_coupled_df__fs: FitSetupTriplet = tag_cal_split_bkg_coupled_df_fit_info.tag_cal_fit_setup
    tc_split_bkg_coupled_d_ds__fs: FitSetupTriplet = tag_cal_split_bkg_coupled_d_ds_fit_info.tag_cal_fit_setup
    #
    # FEI Tag Calibration per B-Tag Reco-Mode
    # Combined BKG Component WITHOUT B -> D l nu UP FEED
    tc_ptm_combined_bkg_coupled_df__fs: FitSetupTriplet = (
        tag_cal_per_tag_mode_combined_bkg_coupled_df_fit_info.tag_cal_fit_setup
    )
    tc_ptm_combined_bkg_floating_ds__fs: FitSetupTriplet = (
        tag_cal_per_tag_mode_combined_bkg_floating_ds_fit_info.tag_cal_fit_setup
    )
    tc_ptm_charge_split_coupled_d__fs: FitSetupTriplet = (
        tag_cal_per_tag_mode_charge_split_coupled_d_fit_info.tag_cal_fit_setup
    )
    tc_ptm_charge_split_floating_ds__fs: FitSetupTriplet = (
        tag_cal_per_tag_mode_charge_split_floating_ds_fit_info.tag_cal_fit_setup
    )
    tc_ptm_b_charge_coupled_d__fs: FitSetupTriplet = tag_cal_per_tag_mode_b_charge_coupled_d_fit_info.tag_cal_fit_setup
    tc_ptm_b_charge_floating_ds__fs: FitSetupTriplet = (
        tag_cal_per_tag_mode_b_charge_floating_ds_fit_info.tag_cal_fit_setup
    )
    #
    # FEI Tag Calibration SigB TagB Closure Test
    tc_closure_a__fs: FitSetupTriplet = tag_cal_closure_a_fit_info.tag_cal_fit_setup
    tc_closure_b__fs: FitSetupTriplet = tag_cal_closure_b_fit_info.tag_cal_fit_setup

    #
    # All D** Calibration Setups
    dss_c_first_approach__fs: FitSetupTriplet = dss_cal_first_approach_fit_info.dss_cal_fit_setup
    dss_c_second_approach__fs: FitSetupTriplet = dss_cal_second_approach_fit_info.dss_cal_fit_setup
    dss_c_third_approach__fs: FitSetupTriplet = dss_cal_third_approach_fit_info.dss_cal_fit_setup

    #
    # Continuum Calibration Setups
    cont_cal_first_approach__fs: FitSetupTriplet = cont_cal_first_approach_fit_info.cont_cal_fit_setup
    cont_cal_second_approach__fs: FitSetupTriplet = cont_cal_second_approach_fit_info.cont_cal_fit_setup
    cont_cal_third_approach__fs: FitSetupTriplet = cont_cal_third_approach_fit_info.cont_cal_fit_setup
    cont_cal_fourth_approach__fs: FitSetupTriplet = cont_cal_fourth_approach_fit_info.cont_cal_fit_setup

    cont_cal_in_b_tag_modes__fs: FitSetupTriplet = cont_cal_in_b_tag_modes_fit_info.cont_cal_fit_setup

    # D** Continuum Calibration Setups
    cont_cal_in_dss_1st_approach__fs: FitSetupTriplet = cont_cal_in_dss_1st_approach_fit_info.cont_cal_fit_setup

    cont_cal_in_b_tag_modes_in_dss__fs: FitSetupTriplet = cont_cal_in_b_tag_modes_in_dss_fit_info.cont_cal_fit_setup
