"""
Selection evaluation infos.

Felix Metzner 2021
"""

from rdstar.offline_analysis.selection.selection_evaluation_utilities import EvalInfoUtils as Eu
from rdstar.offline_analysis.selection.selection_evaluation_information_base import (
    EvalInfoEntry,
    EvalInfoBase,
    create_container_class,
)


__all__ = [
    "BasicEvalInfo",
    "MCEvalInfo",
    "CutAndBCSEvalInfo",
    "MultiplicityEvalInfo",
    "MultiplicityMCEvalInfo",
]


# fmt: off
@create_container_class
class BasicEvalInfo(EvalInfoBase):
    events = EvalInfoEntry(calc=Eu.event_count_fcn(cond_fcn=None), add=Eu.int_add)
    candidates = EvalInfoEntry(calc=Eu.candidate_count_fcn(cond_fcn=None), add=Eu.int_add)

    events_per_data_type = EvalInfoEntry(calc=Eu.static_value_counter(key_fcn=Eu.get_datatype, counter_fcn=Eu.event_count_fcn(cond_fcn=None)), add=Eu.value_counter_add)  # Formerly "data_type_counts_unique"
    candidates_per_data_type = EvalInfoEntry(calc=Eu.static_value_counter(key_fcn=Eu.get_datatype, counter_fcn=Eu.candidate_count_fcn(cond_fcn=None)), add=Eu.value_counter_add)  # Formerly "data_type_counts"

    # Data types
    data_types = EvalInfoEntry(calc=Eu.encountered_values_tuple(values_fcn=Eu.get_datatype), add=Eu.value_tuple_add)

    # Reconstruction modes
    events_per_decay_mode_id = EvalInfoEntry(calc=Eu.event_by_col_counter(col=Eu.decay_mode_id_col, cond_fcn=None), add=Eu.value_counter_add)  # Formerly "unique_events_per_decay_mode_id"
    candidates_per_decay_mode_id = EvalInfoEntry(calc=Eu.candidate_by_col_counter(col=Eu.decay_mode_id_col, cond_fcn=None), add=Eu.value_counter_add)  # Formerly "events_per_decay_mode_id"
    events_per_decay_mode_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.event_by_col_counter, col_y=Eu.decay_mode_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "decay_id_vs_data_type_unique"
    candidates_per_decay_mode_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.candidate_by_col_counter, col_y=Eu.decay_mode_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "decay_id_vs_data_type"

    events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_cond_generator(other_cond_fcn=None)), add=Eu.value_counter_add)
    candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_cond_generator(other_cond_fcn=None)), add=Eu.value_counter_add)

    events_per_tag_decay_mode_id = EvalInfoEntry(calc=Eu.event_by_col_counter(col=Eu.tag_decay_mode_id_col, cond_fcn=None), add=Eu.value_counter_add)  # Formerly "unique_events_per_tag_mode_id"
    candidates_per_tag_decay_mode_id = EvalInfoEntry(calc=Eu.candidate_by_col_counter(col=Eu.tag_decay_mode_id_col, cond_fcn=None), add=Eu.value_counter_add)  # Formerly "events_per_tag_mode_id"
    events_per_tag_decay_mode_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.event_by_col_counter, col_y=Eu.tag_decay_mode_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "tag_id_vs_data_type_unique"
    candidates_per_tag_decay_mode_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.candidate_by_col_counter, col_y=Eu.tag_decay_mode_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "tag_id_vs_data_type"

    events_per_u4s_decay_mode_id = EvalInfoEntry(calc=Eu.event_by_col_counter(col=Eu.u4s_decay_mode_id_col, cond_fcn=None), add=Eu.value_counter_add)  # Formerly "unique_events_per_u4s_mode_id"
    candidates_per_u4s_decay_mode_id = EvalInfoEntry(calc=Eu.candidate_by_col_counter(col=Eu.u4s_decay_mode_id_col, cond_fcn=None), add=Eu.value_counter_add)  # Formerly "events_per_u4s_mode_id"
    events_per_u4s_decay_mode_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.event_by_col_counter, col_y=Eu.u4s_decay_mode_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "u4s_id_vs_data_type_unique"
    candidates_per_u4s_decay_mode_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.candidate_by_col_counter, col_y=Eu.u4s_decay_mode_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "u4s_id_vs_data_type"

    events_per_tag_mode_id_per_decay_mode_id = EvalInfoEntry(calc=Eu.events_by_x_per_y_counter(col_x=Eu.tag_decay_mode_id_col, col_y=Eu.decay_mode_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "u_tag_id_vs_decay_id"
    candidates_per_tag_mode_id_per_decay_mode_id = EvalInfoEntry(calc=Eu.candidates_by_x_per_y_counter(col_x=Eu.tag_decay_mode_id_col, col_y=Eu.decay_mode_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "tag_id_vs_decay_id"

    events_per_u4s_mode_id_per_decay_mode_id = EvalInfoEntry(calc=Eu.events_by_x_per_y_counter(col_x=Eu.u4s_decay_mode_id_col, col_y=Eu.decay_mode_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "u_u4s_id_vs_decay_id"
    candidates_per_u4s_mode_id_per_decay_mode_id = EvalInfoEntry(calc=Eu.candidates_by_x_per_y_counter(col_x=Eu.u4s_decay_mode_id_col, col_y=Eu.decay_mode_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "u4s_id_vs_decay_id"

    events_per_tag_mode_id_per_u4s_mode_id = EvalInfoEntry(calc=Eu.events_by_x_per_y_counter(col_x=Eu.tag_decay_mode_id_col, col_y=Eu.u4s_decay_mode_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "u_tag_id_vs_u4s_id"
    candidates_per_tag_mode_id_per_u4s_mode_id = EvalInfoEntry(calc=Eu.candidates_by_x_per_y_counter(col_x=Eu.tag_decay_mode_id_col, col_y=Eu.u4s_decay_mode_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "tag_id_vs_u4s_id"
# fmt: on


# fmt: off
@create_container_class
class MCEvalInfo(EvalInfoBase):
    # Counts for isSignalAcceptMissing neutrino
    u4s_match_events = EvalInfoEntry(calc=Eu.event_count_fcn(cond_fcn=Eu.u4s_match()), add=Eu.int_add)  # Formerly "n_signal_unique_strict"
    u4s_match_candidates = EvalInfoEntry(calc=Eu.candidate_count_fcn(cond_fcn=Eu.u4s_match()), add=Eu.int_add)  # Formerly "n_signal_strict"

    u4s_match_events_per_decay_mode_id = EvalInfoEntry(calc=Eu.event_by_col_counter(col=Eu.decay_mode_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_counter_add)  # Formerly "is_signal_unique_events_per_decay_mode_id"
    u4s_match_candidates_per_decay_mode_id = EvalInfoEntry(calc=Eu.candidate_by_col_counter(col=Eu.decay_mode_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_counter_add)  # Formerly "is_signal_events_per_decay_mode_id"
    u4s_match_events_per_decay_mode_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.event_by_col_counter, col_y=Eu.decay_mode_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_per_other_counter_add)  # Formerly "is_signal_decay_id_vs_data_type_unique"
    u4s_match_candidates_per_decay_mode_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.candidate_by_col_counter, col_y=Eu.decay_mode_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_per_other_counter_add)  # Formerly "is_signal_decay_id_vs_data_type"

    u4s_match_events_per_tag_decay_mode_id = EvalInfoEntry(calc=Eu.event_by_col_counter(col=Eu.tag_decay_mode_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_counter_add)  # Formerly "is_signal_unique_events_per_tag_mode_id"
    u4s_match_candidates_per_tag_decay_mode_id = EvalInfoEntry(calc=Eu.candidate_by_col_counter(col=Eu.tag_decay_mode_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_counter_add)  # Formerly "is_signal_events_per_tag_mode_id"
    u4s_match_events_per_tag_decay_mode_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.event_by_col_counter, col_y=Eu.tag_decay_mode_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_per_other_counter_add)  # Formerly "is_signal_tag_id_vs_data_type_unique"
    u4s_match_candidates_per_tag_decay_mode_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.candidate_by_col_counter, col_y=Eu.tag_decay_mode_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_per_other_counter_add)  # Formerly "is_signal_tag_id_vs_data_type"

    u4s_match_events_per_u4s_decay_mode_id = EvalInfoEntry(calc=Eu.event_by_col_counter(col=Eu.u4s_decay_mode_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_counter_add)  # Formerly "is_signal_unique_events_per_u4s_mode_id"
    u4s_match_candidates_per_u4s_decay_mode_id = EvalInfoEntry(calc=Eu.candidate_by_col_counter(col=Eu.u4s_decay_mode_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_counter_add)  # Formerly "is_signal_events_per_u4s_mode_id"
    u4s_match_events_per_u4s_decay_mode_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.event_by_col_counter, col_y=Eu.u4s_decay_mode_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_per_other_counter_add)  # Formerly "is_signal_u4s_id_vs_data_type_unique"
    u4s_match_candidates_per_u4s_decay_mode_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.candidate_by_col_counter, col_y=Eu.u4s_decay_mode_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_per_other_counter_add)  # Formerly "is_signal_u4s_id_vs_data_type"

    # Sig ID counts
    events_per_sig_id = EvalInfoEntry(calc=Eu.event_by_col_counter(col=Eu.sig_id_col, cond_fcn=None), add=Eu.value_counter_add)  # Formerly "sig_id_counts_unique"
    candidates_per_sig_id = EvalInfoEntry(calc=Eu.candidate_by_col_counter(col=Eu.sig_id_col, cond_fcn=None), add=Eu.value_counter_add)  # Formerly "sig_id_counts"

    u4s_match_events_per_sig_id = EvalInfoEntry(calc=Eu.event_by_col_counter(col=Eu.sig_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_counter_add)
    u4s_match_candidates_per_sig_id = EvalInfoEntry(calc=Eu.candidate_by_col_counter(col=Eu.sig_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_counter_add)

    events_per_sig_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.event_by_col_counter, col_y=Eu.sig_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)
    candidates_per_sig_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.candidate_by_col_counter, col_y=Eu.sig_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)

    # Sig ID explicit counts (via loop over sig id mc matching queries)
    events_per_sig_id_name_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_counter_add)  # Formerly "n_signal_unique_{sig_id_name}"
    candidates_per_sig_id_name_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_counter_add)  # Formerly "n_signal_{sig_id_name}"
    events_per_sig_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_counter_add)
    candidates_per_sig_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_counter_add)

    events_per_sig_id_name_probe_per_data_type = EvalInfoEntry(calc=Eu.events_multi_condition_by_static_counter(y_key_fcn=Eu.get_datatype, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    candidates_per_sig_id_name_probe_per_data_type = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_static_counter(y_key_fcn=Eu.get_datatype, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    events_per_sig_id_probe_per_data_type = EvalInfoEntry(calc=Eu.events_multi_condition_by_static_counter(y_key_fcn=Eu.get_datatype, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "u_data_type_per_sig_id_{sig_id_info.id}"
    candidates_per_sig_id_probe_per_data_type = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_static_counter(y_key_fcn=Eu.get_datatype, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "data_type_per_sig_id_{sig_id_info.id}"

    events_per_sig_id_name_probe_per_gen_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.gen_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    candidates_per_sig_id_name_probe_per_gen_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.gen_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    events_per_sig_id_probe_per_gen_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.gen_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "u_gen_id_per_sig_id"
    candidates_per_sig_id_probe_per_gen_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.gen_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "gen_id_per_sig_id"

    events_per_sig_id_name_probe_per_decay_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.decay_mode_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    candidates_per_sig_id_name_probe_per_decay_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.decay_mode_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    events_per_sig_id_probe_per_decay_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.decay_mode_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "u_decay_id_vs_sig_id"
    candidates_per_sig_id_probe_per_decay_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.decay_mode_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "decay_id_vs_sig_id"

    events_per_sig_id_name_probe_per_tag_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.tag_decay_mode_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    candidates_per_sig_id_name_probe_per_tag_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.tag_decay_mode_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    events_per_sig_id_probe_per_tag_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.tag_decay_mode_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "u_tag_id_vs_sig_id"
    candidates_per_sig_id_probe_per_tag_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.tag_decay_mode_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "tag_id_vs_sig_id"

    events_per_sig_id_name_probe_per_u4s_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.u4s_decay_mode_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    candidates_per_sig_id_name_probe_per_u4s_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.u4s_decay_mode_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    events_per_sig_id_probe_per_u4s_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.u4s_decay_mode_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "u_u4s_id_vs_sig_id"
    candidates_per_sig_id_probe_per_u4s_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.u4s_decay_mode_id_col, cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "u4s_id_vs_sig_id"

    # Gen ID Counts
    events_per_gen_id = EvalInfoEntry(calc=Eu.event_by_col_counter(col=Eu.gen_id_col, cond_fcn=None), add=Eu.value_counter_add)  # Formerly "gen_id_counts_unique"
    candidates_per_gen_id = EvalInfoEntry(calc=Eu.candidate_by_col_counter(col=Eu.gen_id_col, cond_fcn=None), add=Eu.value_counter_add)  # Formerly "gen_id_counts"

    events_per_gen_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.event_by_col_counter, col_y=Eu.gen_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "gen_id_per_data_type_unique"
    candidates_per_gen_id_per_data_type = EvalInfoEntry(calc=Eu.static_by_x_per_y_counter(x_key_fcn=Eu.get_datatype, counter_fcn=Eu.candidate_by_col_counter, col_y=Eu.gen_id_col, cond_fcn=None), add=Eu.value_per_other_counter_add)  # Formerly "gen_id_per_data_type"

    u4s_match_events_per_gen_id = EvalInfoEntry(calc=Eu.event_by_col_counter(col=Eu.gen_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_counter_add)  # Formerly "gen_id_counts_unique_strict"
    u4s_match_candidates_per_gen_id = EvalInfoEntry(calc=Eu.candidate_by_col_counter(col=Eu.gen_id_col, cond_fcn=Eu.u4s_match()), add=Eu.value_counter_add)  # Formerly "gen_id_counts_strict"

    candidates_with_inconclusive_gen_b1_id = EvalInfoEntry(calc=Eu.candidate_by_col_counter(col=Eu.gen_b1_id_col, cond_fcn=Eu.inconclusive_gen_id()), add=Eu.value_counter_add)  # Formerly "gen_b1_id_for_inconclusive"
    candidates_with_inconclusive_gen_b2_id = EvalInfoEntry(calc=Eu.candidate_by_col_counter(col=Eu.gen_b2_id_col, cond_fcn=Eu.inconclusive_gen_id()), add=Eu.value_counter_add)  # Formerly "gen_b2_id_for_inconclusive"

    # Gen ID explicit counts (via loop over gen id mc matching queries)
    events_per_gen_id_name_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.gen_id_cond_generator(use_name=True)), add=Eu.value_counter_add)
    candidates_per_gen_id_name_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.gen_id_cond_generator(use_name=True)), add=Eu.value_counter_add)
    events_per_gen_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.gen_id_cond_generator(use_name=False)), add=Eu.value_counter_add)
    candidates_per_gen_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.gen_id_cond_generator(use_name=False)), add=Eu.value_counter_add)

    events_per_gen_id_name_probe_per_data_type = EvalInfoEntry(calc=Eu.events_multi_condition_by_static_counter(y_key_fcn=Eu.get_datatype, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    candidates_per_gen_id_name_probe_per_data_type = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_static_counter(y_key_fcn=Eu.get_datatype, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    events_per_gen_id_probe_per_data_type = EvalInfoEntry(calc=Eu.events_multi_condition_by_static_counter(y_key_fcn=Eu.get_datatype, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)
    candidates_per_gen_id_probe_per_data_type = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_static_counter(y_key_fcn=Eu.get_datatype, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)

    events_per_gen_id_name_probe_per_sig_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.sig_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    candidates_per_gen_id_name_probe_per_sig_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.sig_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    events_per_gen_id_probe_per_sig_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.sig_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)
    candidates_per_gen_id_probe_per_sig_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.sig_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)

    events_per_gen_id_name_probe_per_decay_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.decay_mode_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    candidates_per_gen_id_name_probe_per_decay_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.decay_mode_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    events_per_gen_id_probe_per_decay_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.decay_mode_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "u_decay_id_vs_gen_id"
    candidates_per_gen_id_probe_per_decay_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.decay_mode_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "decay_id_vs_gen_id"

    events_per_gen_id_name_probe_per_tag_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.tag_decay_mode_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    candidates_per_gen_id_name_probe_per_tag_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.tag_decay_mode_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    events_per_gen_id_probe_per_tag_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.tag_decay_mode_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "u_tag_id_vs_gen_id"
    candidates_per_gen_id_probe_per_tag_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.tag_decay_mode_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "tag_id_vs_gen_id"

    events_per_gen_id_name_probe_per_u4s_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.u4s_decay_mode_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    candidates_per_gen_id_name_probe_per_u4s_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.u4s_decay_mode_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
    events_per_gen_id_probe_per_u4s_id = EvalInfoEntry(calc=Eu.events_multi_condition_by_col_counter(col=Eu.u4s_decay_mode_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "u_u4s_id_vs_gen_id"
    candidates_per_gen_id_probe_per_u4s_id = EvalInfoEntry(calc=Eu.candidates_multi_condition_by_col_counter(col=Eu.u4s_decay_mode_id_col, cond_fcn_generator=Eu.gen_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "u4s_id_vs_gen_id"
# fmt: on


# fmt: off
@create_container_class
class CutAndBCSEvalInfo(EvalInfoBase):
    # Counts per decay mode id for Y(4S) match and bkg of this match
    u4s_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_cond_generator(other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)  # Formerly "bcs_info_strict_sig"
    u4s_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_cond_generator(other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)  # Formerly "cut_info_strict_sig"
    no_u4s_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_cond_generator(other_cond_fcn=Eu.no_u4s_match())), add=Eu.value_counter_add)
    no_u4s_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_cond_generator(other_cond_fcn=Eu.no_u4s_match())), add=Eu.value_counter_add)  # Formerly "bcs_info_strict_bkg"

    # Counts per decay mode id for sig B match and bkg of this match
    b_sig_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_cond_generator(other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)  # Formerly "bcs_info_strict_b_sig"
    b_sig_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_cond_generator(other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)  # Formerly "cut_info_strict_b_sig"
    no_b_sig_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_cond_generator(other_cond_fcn=Eu.no_b_sig_match())), add=Eu.value_counter_add)
    no_b_sig_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_cond_generator(other_cond_fcn=Eu.no_b_sig_match())), add=Eu.value_counter_add)  # Formerly "bcs_info_strict_b_bkg"

    # Counts per decay mode id for decay_mode to signal_mode match (for sig + norm, sig, norm and bgk; also with additional match of Y(4S) and Bsig)
    reco_sig_or_norm_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_counter_add)  # Formerly "bcs_info_sig"
    reco_sig_or_norm_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_counter_add)  # Formerly "cut_info_sig"
    u4s_match_reco_sig_or_norm_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)  # Formerly "bcs_info_v_strict_sig"
    u4s_match_reco_sig_or_norm_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)  # Formerly "cut_info_v_strict_sig"
    b_sig_match_reco_sig_or_norm_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)  # Formerly "bcs_info_v_strict_b_sig"
    b_sig_match_reco_sig_or_norm_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)  # Formerly "cut_info_v_strict_b_sig"

    reco_sig_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_counter_add)  # Formerly "bcs_info_wn_sig"
    reco_sig_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_counter_add)  # Formerly "cut_info_wn_sig"
    u4s_match_reco_sig_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)  # Formerly "bcs_info_wn_v_strict_sig"
    u4s_match_reco_sig_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)  # Formerly "cut_info_wn_v_strict_sig"
    b_sig_match_reco_sig_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)  # Formerly "bcs_info_wn_v_strict_b_sig"
    b_sig_match_reco_sig_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)  # Formerly "cut_info_wn_v_strict_b_sig"

    reco_norm_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_counter_add)  # Formerly "bcs_info_wn_norm"
    reco_norm_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_counter_add)  # Formerly "cut_info_wn_norm"
    u4s_match_reco_norm_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)  # Formerly "bcs_info_wn_v_strict_norm"
    u4s_match_reco_norm_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)  # Formerly "cut_info_wn_v_strict_norm"
    b_sig_match_reco_norm_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)  # Formerly "bcs_info_wn_v_strict_b_norm"
    b_sig_match_reco_norm_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)  # Formerly "cut_info_wn_v_strict_b_norm"

    reco_bkg_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_counter_add)
    reco_bkg_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_counter_add)  # Formerly "bcs_info_bkg"
    no_u4s_match_reco_bkg_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    no_u4s_match_reco_bkg_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)  # Formerly "bcs_info_v_strict_bkg"
    no_b_sig_match_reco_bkg_match_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)
    no_b_sig_match_reco_bkg_match_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)  # Formerly "bcs_info_v_strict_b_bkg"

    # Counts per decay mode id for decay_mode to signal_mode match with allowing for feed down (for sig + norm, sig, norm and bgk; also with additional match of Y(4S) and Bsig)
    reco_sig_or_norm_match_with_down_feed_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_counter_add)
    reco_sig_or_norm_match_with_down_feed_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_counter_add)
    u4s_match_reco_sig_or_norm_match_with_down_feed_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    u4s_match_reco_sig_or_norm_match_with_down_feed_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    b_sig_match_reco_sig_or_norm_match_with_down_feed_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)
    b_sig_match_reco_sig_or_norm_match_with_down_feed_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)

    reco_sig_match_with_down_feed_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_counter_add)
    reco_sig_match_with_down_feed_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_counter_add)
    u4s_match_reco_sig_match_with_down_feed_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    u4s_match_reco_sig_match_with_down_feed_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    b_sig_match_reco_sig_match_with_down_feed_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)
    b_sig_match_reco_sig_match_with_down_feed_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)

    reco_norm_match_with_down_feed_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_counter_add)
    reco_norm_match_with_down_feed_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_counter_add)
    u4s_match_reco_norm_match_with_down_feed_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    u4s_match_reco_norm_match_with_down_feed_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    b_sig_match_reco_norm_match_with_down_feed_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)
    b_sig_match_reco_norm_match_with_down_feed_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)

    reco_bkg_match_with_down_feed_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_counter_add)
    reco_bkg_match_with_down_feed_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_counter_add)
    no_u4s_match_reco_bkg_match_with_down_feed_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    no_u4s_match_reco_bkg_match_with_down_feed_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    no_b_sig_match_reco_bkg_match_with_down_feed_events_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)
    no_b_sig_match_reco_bkg_match_with_down_feed_candidates_per_decay_mode_id_probe = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)

    # Counts per sig id with matching decay mode id without allowing for feed down
    events_per_reco_sig_id_probe_and_matching_decay_mode = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.sig_id_with_matching_decay_mode_id_condition_generator(allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_counter_add)
    candidates_per_reco_sig_id_probe_and_matching_decay_mode = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.sig_id_with_matching_decay_mode_id_condition_generator(allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_counter_add)
    u4s_match_events_per_reco_sig_id_probe_and_matching_decay_mode = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.sig_id_with_matching_decay_mode_id_condition_generator(allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    u4s_match_candidates_per_reco_sig_id_probe_and_matching_decay_mode = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.sig_id_with_matching_decay_mode_id_condition_generator(allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    b_sig_match_events_per_reco_sig_id_probe_and_matching_decay_mode = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.sig_id_with_matching_decay_mode_id_condition_generator(allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)
    b_sig_match_candidates_per_reco_sig_id_probe_and_matching_decay_mode = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.sig_id_with_matching_decay_mode_id_condition_generator(allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)

    # Counts per sig id with matching decay mode id with allowing for feed down
    events_per_reco_sig_id_probe_and_matching_decay_mode_with_down_feed = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.sig_id_with_matching_decay_mode_id_condition_generator(allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_counter_add)
    candidates_per_reco_sig_id_probe_and_matching_decay_mode_with_down_feed = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.sig_id_with_matching_decay_mode_id_condition_generator(allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_counter_add)
    u4s_match_events_per_reco_sig_id_probe_and_matching_decay_mode_with_down_feed = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.sig_id_with_matching_decay_mode_id_condition_generator(allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    u4s_match_candidates_per_reco_sig_id_probe_and_matching_decay_mode_with_down_feed = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.sig_id_with_matching_decay_mode_id_condition_generator(allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_counter_add)
    b_sig_match_events_per_reco_sig_id_probe_and_matching_decay_mode_with_down_feed = EvalInfoEntry(calc=Eu.events_multi_condition_counter(cond_fcn_generator=Eu.sig_id_with_matching_decay_mode_id_condition_generator(allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)
    b_sig_match_candidates_per_reco_sig_id_probe_and_matching_decay_mode_with_down_feed = EvalInfoEntry(calc=Eu.candidates_multi_condition_counter(cond_fcn_generator=Eu.sig_id_with_matching_decay_mode_id_condition_generator(allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_counter_add)
# fmt: on


# fmt: off
@create_container_class
class MultiplicityEvalInfo(EvalInfoBase):
    event_multiplicities = EvalInfoEntry(calc=Eu.multiplicity_counter(cond_fcn=None), add=Eu.value_counter_add)  # Formerly "event_multiplicities"
    event_multiplicities_per_data_type = EvalInfoEntry(calc=Eu.static_value_per_other_counter(key_fcn=Eu.get_datatype, counter_fcn=Eu.multiplicity_counter(cond_fcn=None)), add=Eu.value_per_other_counter_add)  # Formerly "event_multiplicities_per_data_type"
    event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_cond_generator(other_cond_fcn=None)), add=Eu.value_per_other_counter_add)  # Formerly "event_multiplicities_per_decay_id"
# fmt: on


# fmt: off
@create_container_class
class MultiplicityMCEvalInfo(EvalInfoBase):
    # Event multiplicities per decay id and matching sig id
    reco_sig_or_norm_match_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_per_other_counter_add)  # Formerly "event_multiplicities_is_sig_per_decay_id"
    reco_sig_match_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_per_other_counter_add)
    reco_norm_match_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_per_other_counter_add)
    reco_bkg_match_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=False, other_cond_fcn=None)), add=Eu.value_per_other_counter_add)

    u4s_match_reco_sig_or_norm_match_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_per_other_counter_add)  # Formerly "event_multiplicities_is_sig_strict_per_decay_id"
    u4s_match_reco_sig_match_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_per_other_counter_add)
    u4s_match_reco_norm_match_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_per_other_counter_add)
    no_u4s_match_reco_bkg_match_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=False, other_cond_fcn=Eu.u4s_match())), add=Eu.value_per_other_counter_add)

    b_sig_match_reco_sig_or_norm_match_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_per_other_counter_add)  # Formerly "event_multiplicities_is_sig_strict_b_per_decay_id"
    b_sig_match_reco_sig_match_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_per_other_counter_add)
    b_sig_match_reco_norm_match_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_per_other_counter_add)
    no_b_sig_match_reco_bkg_match_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=False, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_per_other_counter_add)

    # Event multiplicities per decay id and matching sig id with allowing for feed down
    reco_sig_or_norm_match_with_down_feed_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_per_other_counter_add)
    reco_sig_match_with_down_feed_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_per_other_counter_add)
    reco_norm_match_with_down_feed_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_per_other_counter_add)
    reco_bkg_match_with_down_feed_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=True, other_cond_fcn=None)), add=Eu.value_per_other_counter_add)

    u4s_match_reco_sig_or_norm_match_with_down_feed_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_per_other_counter_add)
    u4s_match_reco_sig_match_with_down_feed_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_per_other_counter_add)
    u4s_match_reco_norm_match_with_down_feed_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_per_other_counter_add)
    no_u4s_match_reco_bkg_match_with_down_feed_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=True, other_cond_fcn=Eu.u4s_match())), add=Eu.value_per_other_counter_add)

    b_sig_match_reco_sig_or_norm_match_with_down_feed_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig_plus_norm", allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_per_other_counter_add)
    b_sig_match_reco_sig_match_with_down_feed_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="sig", allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_per_other_counter_add)
    b_sig_match_reco_norm_match_with_down_feed_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="norm", allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_per_other_counter_add)
    no_b_sig_match_reco_bkg_match_with_down_feed_event_multiplicities_per_decay_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.decay_mode_id_with_sig_match_cond_generator(version="bkg", allow_down_feed=True, other_cond_fcn=Eu.b_sig_match())), add=Eu.value_per_other_counter_add)

    # Event multiplicities per sig id
    event_multiplicities_per_sig_id = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.sig_id_cond_generator(use_name=False)), add=Eu.value_per_other_counter_add)  # Formerly "event_multiplicities_per_sig_id"
    event_multiplicities_per_sig_id_name = EvalInfoEntry(calc=Eu.multiplicity_multi_condition_counter(cond_fcn_generator=Eu.sig_id_cond_generator(use_name=True)), add=Eu.value_per_other_counter_add)
# fmt: on
